/*
 * ubus_call.c -
 *
 * Copyright (C) 2020 iopsys Software Solutions AB. All rights reserved.
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include "ubus_call.h"
#include <time.h>

#define UBUS_DEFAULT_TIMEOUT 1500
#define UBUS_DEFAULT_POLL_10_SEC 10000

struct ubus_context *ctx;

int ubus_init(void) {

  uloop_init();
  ctx = ubus_connect(NULL);

  if (!ctx) {
    printf("failed to connect to UBUS");
    goto __ret;
  }

  return EXIT_SUCCESS;

__ret:
  ubus_free(ctx);
  uloop_done();
  return EXIT_FAILURE;
}

void parse_ubus_cb(struct ubus_request *req, int type, struct blob_attr *msg) {

  struct json_object *root;
  struct json_object **json_ubus_result = req->priv;
  char *json_str;

  json_str = blobmsg_format_json(msg, true);
  if (!json_str)
    return;

  root = json_tokener_parse(json_str);

  if (!root)
    return;

  *json_ubus_result = json_object_get(root);

  json_object_put(root);
  free(json_str);
}

struct ubus_result *ubus_call(const char *object, const char *method,
                              const char *args) {

  struct blob_buf buff = {0};
  int rv = -1;
  uint32_t obj_id;
  struct json_object *json_ubus_result;
  struct ubus_result *res = NULL;

  ubus_lookup_id(ctx, object, &obj_id);

  if (!obj_id) {
    fprintf(stderr, "ubus lookup_id error \n");
    return NULL;
  }

  rv = blob_buf_init(&buff, 0);

  if (rv) {
    fprintf(stderr, "blob_buf_init error \n");
    return NULL;
  }

  if (args) {
    rv = blobmsg_add_json_from_string(&buff, args);
    if (!rv) {
      perror("could not convert arg to json \n");
      goto out;
    }
  }

  rv = ubus_invoke(ctx, obj_id, method, buff.head,
                   (ubus_data_handler_t)parse_ubus_cb, &json_ubus_result,
                   UBUS_DEFAULT_TIMEOUT);

  res = calloc(1, sizeof(struct ubus_result));
  if (!res)
    goto out;

  res->rc = rv;
  res->result = json_ubus_result;

out:
  blob_buf_free(&buff);

  return res;
}